{*******************************************************}
{                                                       }
{    LocaleDialog                                    	}
{    Version 1.0                                     	}
{    Copyright  1998 by DithoSoft Software Solutions	}
{                                                       }
{*******************************************************}
unit LocaleDlg;
{$R LOCALEDLG.RES}

{
  TLocaleDialog  is  a dialog  component that  allows  you  to  select
  from the  Windows language codes. Each  language code is represented
  by a string (e.g. "English (USA)"). You can also obtain an "official"
  abbreviation for the selected language (e.g. "enu" -> "English (USA)").

  TLocaleDialog  is useful in any  application where  it is required
  to select a language. Please note  that nothing but  the code page
  (e.g. $0409 -> "English (USA)"), the language description  and the
  abbreviated  language name  is returned. You have to do everything
  else!

  TLocaleDialog also shows how to create dialogs at runtime and how
  to create an owner drawn list box.

  You can download the latest version of this component under
      http://dithosoft.cjb.net

  You are welcome to send questions, suggestions or comments to
      thorsten.dittmar@debitel.net
}


// Remove the "#" to use this component with Delphi 2
{#$DEFINE DELPHI2}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  { TLocaleOptions }
  TLocaleOption = (loInstructions, loHelpButton);
  TLocaleOptions= set of TLocaleOption;

  { TLocaleDialog }
  TLocaleDialog = class(TComponent)
  private
    FLocale              : LongInt;
    FLocaleDescription,
    FLocaleShortName     : String;
    FOptions             : TLocaleOptions;
    FInstructions,
    FTitle               : String;
    FOnShowDialog,
    FOnHideDialog,
    FOnHelp              : TNotifyEvent;

    procedure   SetInstructions(Value: String);
    procedure   SetOptions(Value: TLocaleOptions);
    procedure   SetLocale(Value: LongInt);
    procedure   SetTitle(Value: String);
    function    GetDefaultLocale: LongInt;
    function    GetDefaultLocaleShortName: String;
    function    GetDefaultLocaleDescription: String;
    function    GetLocale(const ALocaleDescription: String): LongInt;
    function    GetLocaleShortName(const ALocale: LongInt): String;
    function    GetLocaleDescription(const ALocale: LongInt): String;
  protected
    function    DoExecute: Boolean;
    procedure   FillLocaleList(ADlg: TForm);
  public
    constructor Create(AOwner: TComponent); override;
    function    Execute: Boolean;

    property    DefaultLocale: LongInt read GetDefaultLocale;
    property    DefaultLocaleDescription: String read GetDefaultLocaleDescription;
    property    DefaultLocaleShortName: String read GetDefaultLocaleShortName;
    property    LocaleDescription: String read FLocaleDescription;
    property    LocaleShortName: String read FLocaleShortName;
  published
    property    Instructions: String read FInstructions write SetInstructions;
    property    Locale: LongInt read FLocale write SetLocale;
    property    Options: TLocaleOptions read FOptions write SetOptions;
    property    Title: String read FTitle write SetTitle;

    property    OnHelp: TNotifyEvent read FOnHelp write FOnHelp;
    property    OnHideDialog: TNotifyEvent read FOnHideDialog write FOnHideDialog;
    property    OnShowDialog: TNotifyEvent read FOnShowDialog write FOnShowDialog;
  end;

procedure Register;

implementation

uses
  Consts, StdCtrls, ExtCtrls, Registry;

const
  LocaleParentKey = 'System\CurrentControlSet\Control\Nls\Locale';

{
  The following dialog template is used to be able to access components that are
  created at runtime without having to use  "FindComponent". Also,  the template
  can manager owner drawing itself.  It loads a bitmap from the resource as soon
  as it is created and measures/draws the items when needed.
}
type
  { TSelectDlg template }
  TSelectDlg = class(TForm)
    ListBox      : TListBox;
    Separator    : TBevel;
    Instruction  : TLabel;
    OKButton,
    CancelButton,
    HelpButton   : TButton;
  private
    FListImage   : TBitmap;
    procedure ListDraw(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure ListMeasure(Control: TWinControl; Index: Integer; var Height: Integer);
  public
    constructor CreateNew(AOwner: TComponent);
    destructor  Destroy; override;
  end;

{ ------------------------------------------------------------------------------------------------
  Component Registration
  ------------------------------------------------------------------------------------------------ }
procedure Register;
begin
  { Register the component }
  RegisterComponents('Dialogs', [TLocaleDialog]);
end;


{ ------------------------------------------------------------------------------------------------
  Tools
  ------------------------------------------------------------------------------------------------ }
function HexToDec(const Value: String): Integer;
var s: String;
begin
  { Convert a hex number to a string making sure there is a leading $-sign }
  s := Value;
  if Pos('$',s) = 0 then s := '$'+Value;
  Result := StrToIntDef(s,0);
end;



{ ------------------------------------------------------------------------------------------------
  TSelectDlg template
  ------------------------------------------------------------------------------------------------ }
constructor TSelectDlg.CreateNew(AOwner: TComponent);
begin
  inherited CreateNew(AOwner);
  FListImage := TBitmap.Create;
  FListImage.LoadFromResourceName(HInstance,'BMP_LOCALE_DLG');
end;

destructor TSelectDlg.Destroy;
begin
  FListImage.Free;
  inherited Destroy;
end;

procedure TSelectDlg.ListDraw(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var SrcRect,DstRect: TRect;
    x,y            : Integer;
    AText          : String;
begin
  with ListBox.Canvas do begin
    { Fill the rectangle with the active color (selected/normal) }
    FillRect(Rect);

    { Draw the image transparently }
    SetRect(SrcRect,0,0,FListImage.Width,FListImage.Height);
    DstRect.Top := Rect.Top+((Rect.Bottom-Rect.Top-FListImage.Height) div 2);
    DstRect.Left := Rect.Left+2;
    DstRect.Right := DstRect.Left+FListImage.Width;
    DstRect.Bottom := DstRect.Top+FListImage.Height;
    BrushCopy(DstRect,FListImage,SrcRect,clFuchsia);

    { Draw the text }
    AText := ListBox.Items[Index];
    y     := Rect.Top+((Rect.Bottom-Rect.Top-TextHeight(AText)) div 2);
    x     := DstRect.Left+DstRect.Right+4;
    TextOut(x,y,AText);
  end;
end;

procedure TSelectDlg.ListMeasure(Control: TWinControl; Index: Integer; var Height: Integer);
begin
  { Return the height of a list item }
  Height := Abs(Font.Height)+5;
  if Height < FListImage.Height+4 then Height := FListImage.Height+4;
end;



{ ------------------------------------------------------------------------------------------------
  TLocaleDialog
  ------------------------------------------------------------------------------------------------ }
constructor TLocaleDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  { Initialize properties }
  FInstructions       := '';
  FOptions            := [];
  FLocale             := $0;
  FLocaleDescription  := '';
  FLocaleShortName    := '';
  FTitle              := '';
end;

function TLocaleDialog.Execute: Boolean;
begin
  { Execute the dialog }
  Result := DoExecute;
end;

function TLocaleDialog.DoExecute: Boolean;
const Heights: Array[Boolean] of Integer = (152,201);
var Dlg  : TSelectDlg;
begin
  { Manually create the dialog }
  Dlg := TSelectDlg.CreateNew(Application);
  try
    { Set the properties of the dialog form }
    with Dlg do begin
      AutoScroll    := False;
      BorderIcons   := [biSystemMenu];
      BorderStyle   := bsDialog;
      Caption       := Title;
      if Caption = '' then Caption := 'Select language';
      ClientHeight  := Heights[(loInstructions in Options) and (Instructions <> '')];
      ClientWidth   := 321;
      Name          := 'LocaleDialog_SelectDlg';
      Position      := poScreenCenter;
      OnShow        := FOnShowDialog;
      OnHide        := FOnHideDialog;

      { Insert the ListBox }
      ListBox       := TListBox.Create(Dlg);
      with ListBox do begin
        Parent        := Dlg;
        BorderStyle   := bsSingle;
        Left          := 8;
        Top           := 8;
        Width         := 217;
        Height        := 137;
        Style         := lbOwnerDrawVariable;
        Sorted        := True;
        OnMeasureItem := ListMeasure;
        OnDrawItem    := ListDraw;
      end;

      { Insert the Separator }
      Separator     := TBevel.Create(Dlg);
      with Separator do begin
        Parent      := Dlg;
        Height      := 3;
        Left        := 8;
        Shape       := bsTopLine;
        Top         := 152;
        Width       := 305;
      end;

      { Insert the Instruction }
      Instruction   := TLabel.Create(Dlg);
      with Instruction do begin
        Parent      := Dlg;
        Alignment   := taCenter;
        AutoSize    := False;
        Caption     := Instructions;
        WordWrap    := True;
        Top         := 160;
        Left        := 8;
        Height      := 35;
        Width       := 305;
      end;

      { Insert the OK Button }
      OKButton      := TButton.Create(Dlg);
      with OKButton do begin
        Parent      := Dlg;
        Caption     := {$IFDEF DELPHI2}LoadStr{$ENDIF}(SMsgDlgOK);
        Default     := True;
        Height      := 23;
        Left        := 232;
        ModalResult := mrOK;
        Top         := 8;
        Width       := 81;
      end;

      { Insert the Cancel Button }
      CancelButton  := TButton.Create(Dlg);
      with CancelButton do begin
        Parent      := Dlg;
        Cancel      := True;
        Caption     := {$IFDEF DELPHI2}LoadStr{$ENDIF}(SMsgDlgCancel);
        Height      := 23;
        Left        := 232;
        ModalResult := mrCancel;
        Top         := 32;
        Width       := 81;
      end;

      { Insert the Help Button }
      HelpButton    := TButton.Create(Dlg);
      with HelpButton do begin
        Parent      := Dlg;
        Caption     := {$IFDEF DELPHI2}LoadStr{$ENDIF}(SMsgDlgHelp);
        Height      := 23;
        Left        := 232;
        Width       := 81;
        Top         := 64;
        ModalResult := mrNone;
        OnClick     := FOnHelp;
        Visible     := loHelpButton in Options;
      end;
    end;

    { Fill the list box and select the specified/current locale string }
    FillLocaleList(Dlg);

    Result := Dlg.ShowModal = mrOK;
    if Result then begin
      { Set the properties to represent the selected locale }
      FLocaleDescription := Dlg.ListBox.Items[Dlg.ListBox.ItemIndex];
      FLocale            := GetLocale(FLocaleDescription);
      FLocaleShortName   := GetLocaleShortName(FLocale);
    end;
  finally
    Dlg.Free;
  end;
end;

procedure TLocaleDialog.FillLocaleList(ADlg: TForm);
var Reg   : TRegistry;
    Dlg   : TSelectDlg;
    Values: TStringList;
    Value : String;
    Loc   : LongInt;
    i     : Integer;
begin
  { Read all the valid locales from the Registry }
  if not (ADlg is TSelectDlg) then exit;
  Dlg := ADlg as TSelectDlg;

  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(LocaleParentKey,False) then begin

      { Create a list of all values in this key }
      Values := TStringList.Create;
      try
        Reg.GetValueNames(Values);

        { Iterate through all the values }
        for i := 0 to Values.Count-1 do begin
          if Values[i] <> '' then begin
            Value := Reg.ReadString(Values[i]);
            if Value <> '' then Dlg.ListBox.Items.Add(Value);
          end;
        end;

        { Select the default locale or the locale specified by FLocale }
        if FLocale <> $0 then Loc := FLocale else Loc := GetDefaultLocale;
        for i := 0 to Dlg.ListBox.Items.Count-1 do
          if Dlg.ListBox.Items[i] = GetLocaleDescription(Loc) then begin
            Dlg.ListBox.ItemIndex := i;
            Break;
          end;
      finally
        Values.Free;
      end;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TLocaleDialog.SetInstructions(Value: String);
begin
  if FInstructions <> Value then
    FInstructions := Value;
end;

procedure TLocaleDialog.SetOptions(Value: TLocaleOptions);
begin
  if FOptions <> Value then
    FOptions := Value;
end;

procedure TLocaleDialog.SetLocale(Value: LongInt);
begin
  if FLocale <> Value then
    FLocale := Value;
end;

procedure TLocaleDialog.SetTitle(Value: String);
begin
  if FTitle <> Value then
    FTitle := Value;
end;

function TLocaleDialog.GetDefaultLocale: LongInt;
var Reg: TRegistry;
begin
  { Return the code of the default locale }
  Result := 0;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(LocaleParentKey,False) then Result := HexToDec(Reg.ReadString(''));
  finally
    Reg.Free;
  end;
end;

function TLocaleDialog.GetDefaultLocaleShortName: String;
begin
  { Return the abbreviated language name of the default locale }
  Result := GetLocaleShortName(GetDefaultLocale);
end;

function TLocaleDialog.GetDefaultLocaleDescription: String;
begin
  { Return the description of the default locale }
  Result := GetLocaleDescription(GetDefaultLocale);
end;

function TLocaleDialog.GetLocale(const ALocaleDescription: String): LongInt;
var Reg: TRegistry;
    s  : TStringList;
    i  : Integer;
begin
  { Return the locale that belongs to a description }
  Result := 0;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(LocaleParentKey,False) then begin
      s := TStringList.Create;
      try
        Reg.GetValueNames(s);
        for i := 0 to s.Count-1 do begin
          if Reg.ReadString(s[i]) = ALocaleDescription then begin
            Result := HexToDec(s[i]);
            Break;
          end;
        end;
      finally
        s.Free;
      end;
    end;
  finally
    Reg.Free;
  end;
end;

function TLocaleDialog.GetLocaleShortName(const ALocale: LongInt): String;
var Buffer : Array[0..10] of Char;
begin
  { Return the abbreviated language name of a locale }
  Result  := '';
  if GetLocaleInfo(ALocale,LOCALE_SABBREVLANGNAME,Buffer,10) <> 0 then
    Result := StrPas(Buffer);
end;

function TLocaleDialog.GetLocaleDescription(const ALocale: LongInt): String;
var Reg: TRegistry;
    s  : TStringList;
    i  : Integer;
begin
  { Return the locale description of a locale }
  Result := '';
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey(LocaleParentKey,False) then begin
      s := TStringList.Create;
      try
        Reg.GetValueNames(s);
        for i := 0 to s.Count-1 do begin
          if HexToDec(s[i]) = ALocale then begin
            Result := Reg.ReadString(s[i]);
            Break;
          end;
        end;
      finally
        s.Free;
      end;
    end;
  finally
    Reg.Free;
  end;
end;

end.
